.template-tile {
    background-color: #CCCCCC;
    height: 100%;
}

/***** astro-viewer *****/
.astro-viewer-ctrl-astro-btn {
    background-color: #FF4D4D;
}
.astro-viewer-ctrl-visibility{
    background-position:center;
    background-repeat:no-repeat;
    background-size:80%;
    height: 15px;
    width: 15px;
    cursor:pointer;
}
.astro-viewer-ctrl-invisible {
    background-image:url(/toyz/static/astrotoyz/icons/invisible.png);
}
.astro-viewer-catalog-ctrl {
    float: left;
}