# Core classes and functions for Astro-Toyz
# Copyright 2015 by Fred Moolekamp
# License: LGPLv3
from toyz.utils.errors import ToyzError

class AstroToyzError(ToyzError):
    """
    Class for errors initiating in Astro Toyz
    """
    pass