<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
        <title>Sextractor GUI</title>
        <script src="/third_party/jquery/jquery.min.js" type="text/javascript"></script>
        <script src="/static/web/static/toyz_core.js" type="text/javascript"></script>
        <script src="/toyz/static/astrotoyz/sextractor.js" type="text/javascript"></script>
    </head>
    <body>
        <button id='run-btn'>Click to run SExtractor</button>
        <script>
        var sex_interface;
        var run_btn = $('#run-btn');
        var websocket;
        window.onload = function(){
            websocket = new Toyz.Core.Websocket();
            Toyz.Astro.Sex.load_dependencies(function(){
                sex_interface = new Toyz.Astro.Sex.Interface();
            });
            run_btn.click(function(){
                sex_interface.config.$div.dialog('open');
            });
        };
        </script>
    </body>
</html>